 function [vec, val, err] = arn0 (A,m,v,lam)
%--------------------------------------------
% function [vec, val, err] = arn0 (A,m,v,lam)
% m steps of the Arnoldi process:
% A   = sparse matrix
% m   = dimension of Krylov subspace
% v   = initial vector,
% lam = seek eigenvalue closest to lam
% return: 
% vec = approximate eigenvector
% val = approximate eigenvalue
% err = error history
 %--------------------------------------------
 n = size(A,1);
%--------------------------------------------
% start procedure
%-------------------------------------------- 
 ro = norm(v,2)  ; 
 t = 1.0/ ro   ;
 V(1:n,1) = v*t; 
%% 
for i=1:m  
  i1 = i + 1  ; 
  V(1:n,i1) = A*V(1:n,i)  ;
%--------------------------------------------
%                 modified GS  ;
%--------------------------------------------
 for j=1:i 
   t = V(1:n,j)'*V(1:n,i1)  ;
   H(j,i) = t  ;
   V(1:n,i1) = V(1:n,i1) - t*V(1:n,j)  ; 
 end  ;
%
 t = norm(V(1:n,i1),2)  ;
 H(i1,i) = t  ;
 if (t  ~= 0.0)  
   t = 1.0 / t  ;
   V(1:n,i1) = V(1:n,i1)*t  ;  
 end  
%--------------------------------------------
% define Arnoldi Objects 
%--------------------------------------------
 Vm = V(1:n,1:i);
 Hm = H(1:i,1:i);
%%%%%%%%%%%%%%%%%%%%%%%%%%%% Pure arnoldi 
 [X,vals] = eig(Hm);
 vals = diag(vals);
 [er,k]  = min(abs(lam - vals));
 err(i) = er;
end
val = vals(k) ;
vec = Vm*X(:,k);

