  function [W, R, res] = arnRD(A,m,v,nev,order,tol,maxits)
% function [W, R, res] = arnR(A,v,m,nev,order,tol,maxits)
% restarted  Arnoldi process with deflation.
% m = dimension of Krylov subspace
% order = 'ascend' or 'descend' -- computes eigenvalues of
% smallest (resp. largest)  real parts
% tol   = tolerance for stopping criterion
% maxits= max number of (inner) Arnoldi steps;
%
%  OUT:
%  W = sets of schur vectors 
%  R = related Schur form 
%  res  = residual norm history
%%--------------------
W = [];
%%R = zeros(m,m);
R = []; 
 n = size(A,1);
 res = 1.0;  tol1 = 0.0;
%%-------------------- restart loop 
 nevc = 0;
 ro = norm(v)  ; 
%%-------------------- if ro == 0 ...
 if (ro == 0), error('zero vector for v'), end
 V(:,1) = v/ro;
 its = 0;
 while (its < maxits) 
    its = its+1;
  %% restart 
  %%-------------------- anoldi loop 
  for i=1:m  
    i1 = i + 1  ;
    v  = A*V(:,i)  ;
    %%-------------------- Deflation step 
    if  (nevc) 
      v = v - W*(W'*v);
    end
    V(:,i1) = v;
%%-------------------- modified GS  ;
    for j=1:i 
      t = V(:,j)'*V(:,i1)  ;
      H(j,i) = t  ;
      V(:,i1) = V(:,i1) - t*V(:,j)  ; 
    end 
%%
   t = norm(V(:,i1),2)  ;
   H(i1,i) = t  ;
   if (t == 0.0),break,end
   t = 1.0 / t  ;
   V(:,i1) = V(:,i1)*t  ;  
  end
  %%-------------------- Arnoldi Objects 
  Vm = V(:,1:i);
  Hm = H(1:i,1:i);
%%--------------------  get Ritz values and vectors
  [X,vals] = eig(Hm);
  vals = diag(vals);
  [~,indx]=sort(real(vals),order);
%%  [t, k] = max(order*real(vals));
  k =  indx(1);
  v = V(:,1:i)*X(:,k) ;
%%
  Av = A*v;
%%-------------------- nevc = # converged eigenvalues --   
  if  (nevc >0) 
    %%%%% matrix R IS NOT NEEDED
    %%   Av = Av - W*(R*W'*Av);
       Av = Av - W*(W'*Av);
  end
%%--------------------
  lam = v'*Av;
  ro = norm(Av - lam*v);
  if (its == 1) 
    tol1 = tol*res;
  end
  res(its)  = ro;
  if (ro < tol1) 
    nevc = nevc+1;
    if (nevc >1) 
      t = W'*v;
      v = v - W*t;
      v = v / norm(v);
    end
    nevc
%%-------------------- store v
    W(:,nevc) = v;
    R(1:nevc,nevc) = W'*(A*v);
%%-------------------- reset v for restarting
    k =  indx(2);
    v = V(:,1:i)*X(:,k) ;
    v = v/norm(v);
%%-------------------- trick for showing separate curves
    its = its+1;
  res(its)= nan;
  end
  if (nevc >= nev);
    disp(' ----------converged ')
    [nevc, nev]
    return
  end
  %%-------------------- else restart
  V(:,1) = v;
end
