  function [B, XN, Pr, wo] = coarsestep2(D, X, wx)
% function [B, XN, Pr, wo] = coarsestep2(D, X, wx)   
% Algorithm proceeds by traversing EDGES - based on D
%%    (largest to smallest weights in D)     
% IN
%  D   = weight matrix..  
%  X   = data matrix - each columns contains a point (e.g, xyz
%        coordinates or some other features);
%  wx  = weights of vertices [row vector] [1,n] 
%
% OUT: 
%  B    = Output sparse  matrix of weights - may be used for next level
%  XN   = coarse nodes (coarsened `features')
%        in coarse graph 
% wo    = weights for output nodes  [row vector]
%  Pr   = Projection matrix used to produce B, wo,  and XN:
%          [B = Pr'*D*Pr;  XN = X*Pr;  wo = wx*Pr; ]
%% ASSUMES DIAGONAL OF D IS ZERO.

N = size(D,1);     % Numbers of nodes of the given (fine) graph.
prnt = zeros(N,1); % parents list 
%%-------------------- unroll sparse matrix into arrays.
[I, J, W] = find(D);
nz = length(J);

[~, indx] = sort(W,'descend');

%%[dummy, indx] = sort(W,'descend');
%%indx(1:200)
%%%indx = randperm(nz) ;
new = 0;
%%indx = randperm(nz);
kp = 0;
for k = 1:nz 
  next = indx(k);
  i = I(next);
  if (prnt(i) >0), continue, end;% skip if parent assigned
  j = J(next);
%%-------------------- exclude i->i links
  if (i <= j), continue, end;
  if (prnt(j) >0), continue, end;
%%-------------------- create new node 
  new = new+1;
%%disp(new)
%%-------------------- parents of this new node 
  prnt(i) = new;
  prnt(j) = new;
%%-------------------- projection matrix
%%  1st
  kp = kp+1;
  ip(kp) = i;
  jp(kp) = new;
%% 2nd
  kp = kp+1;
  ip(kp) = j;
  jp(kp) = new;
end
%%-------------------- mop up - left-over singletons -- 
for ii=1:N
    if (prnt(ii) ~= 0), continue, end 
%%-------------------- real singleton -- zero row  
    if (nnz(D(:,ii)) ==0) 
      new = new+1;
      prnt(ii) = new;
%%-------------------- update projection matrix:
      kp = kp+1;
      ip(kp) = ii;
      jp(kp) = new;
    else
%%-------------------- find best candidate among neighbors to add as parent
      [~, idx] = max(D(:,ii));
      prnt(ii) = new; %% prnt(idx);
%%       kp = kp+1;
%%       ip(kp) = ii;
%%       jp(kp) = new;  %%prnt(idx);
%%------------------- update projection matirx:
    end
end
%%-------------------- now build the projection graph;
Pr = sparse(ip, jp, 1, N, new);
for j=1:new
  [ir, ~, ~] = find(Pr(:,j));
  %% t = norm(wx(ir));
  t = sum(wx(ir));
  Pr(ir,j)  = wx(ir)'/t;
end
%%-------------------- reorder Pr to best preserve original structure 
for j=1:new
  iir(j) = find(Pr(:,j),1,'first');  
  %%iir(j) = min(find(Pr(:,j))); 
end
[~, idx] = sort(iir);
Pr = Pr(:,idx);
%%-------------------- return modified objects
XN = X*Pr;
wo = wx*Pr;
B = Pr'*D*Pr;
